@@ -1,8 +1,8 @@
 /*
-** Copyright 1998 - 2005 Double Precision, Inc.
+** Copyright 1998 - 2010 Double Precision, Inc.
 ** See COPYING for distribution information.
 **
-** $Id: pop3dserver.c,v 1.38 2005/10/01 03:23:44 mrsam Exp $
+** $Id: pop3dserver.c,v 1.45 2010/03/19 01:09:26 mrsam Exp $
 */
 
 #if	HAVE_CONFIG_H
@@ -62,7 +62,7 @@
 extern void pop3dcapa();
 static void acctout(const char *disc);
 
-static const char *authaddr, *remoteip;
+static const char *authaddr, *remoteip, *remoteport;
 
 struct msglist {
 	struct msglist *next;
@@ -524,8 +524,8 @@ static void cleanup()
 {
 	unsigned i;
 
-	long deleted_bytes=0;
-	long deleted_messages=0;
+	int64_t deleted_bytes=0;
+	int64_t deleted_messages=0;
 
 	for (i=0; i<msglist_cnt; i++)
 		if (msglist_a[i]->isdeleted)
@@ -794,6 +794,7 @@ static void acctout(const char *disc)
 {
 	static const char msg2[]=", user=";
 	static const char msg3[]=", ip=[";
+	static const char msgport[]="], port=[";
 	static const char msg4[]="], top=";
 	static const char msg5[]=", retr=";
 	static const char msg6[]=", time=";
@@ -816,7 +817,7 @@ static void acctout(const char *disc)
 	libmail_str_size_t(bytes_received_count, numAR);
 	libmail_str_size_t(bytes_sent_count, numAS);
 
-	p=malloc(strlen(authaddr)+strlen(remoteip)+strlen(disc)+
+	p=malloc(strlen(authaddr)+strlen(remoteip)+strlen(remoteport)+strlen(disc)+
 		 strlen(num1)+strlen(num2)+strlen(num3)+
 		 strlen(numAR)+strlen(numAS)+200);	/* Should be enough */
 
@@ -825,6 +826,8 @@ static void acctout(const char *disc)
 	strcat(p, authaddr);
 	strcat(p, msg3);
 	strcat(p, remoteip);
+	strcat(p, msgport);
+	strcat(p, remoteport);
 	strcat(p, msg4);
 	strcat(p, num1);
 	strcat(p, msg5);
@@ -836,11 +839,12 @@ static void acctout(const char *disc)
 	strcat(p, msg6);
 	strcat(p, num3);
 
-	if ((q=getenv("POP3_STLS")) && atoi(q))
+	if ((q=getenv("POP3_TLS")) && atoi(q))
 		strcat(p, msg7);
 
 	strcat(p, "\n");
-	write(2, p, strlen(p));
+	if (write(2, p, strlen(p)) < 0)
+		; /* make gcc shut up */
 	free(p);
 }
 
@@ -1006,6 +1010,9 @@ int main(int argc, char **argv)
 {
 char	*p;
 
+#ifdef HAVE_SETVBUF_IOLBF
+	setvbuf(stderr, NULL, _IOLBF, BUFSIZ);
+#endif
 	time(&start_time);
 
 	if ((authaddr=getenv("AUTHADDR")) == NULL ||
@@ -1019,13 +1026,16 @@ char	*p;
 	if ((remoteip=getenv("TCPREMOTEIP")) == NULL)
 		remoteip="127.0.0.1";
 
+	if ((remoteport=getenv("TCPREMOTEPORT")) == NULL)
+		remoteport="0";
+
 	{
 	struct	stat	buf;
 
 		if ( stat(".", &buf) < 0 || buf.st_mode & S_ISVTX)
 		{
-			fprintf(stderr, "INFO: LOCKED, user=%s, ip=[%s]\n",
-				authaddr, remoteip);
+			fprintf(stderr, "INFO: LOCKED, user=%s, ip=[%s], port=[%s]\n",
+							authaddr, remoteip, remoteport);
 			printed(printf("-ERR Your account is temporarily unavailable (+t bit set on home directory).\r\n"));
 			exit(0);
 		}
@@ -1056,9 +1066,18 @@ char	*p;
 		exit(1);
 	}
 
-	fprintf(stderr, "INFO: LOGIN, user=%s, ip=[%s]\n",
+	if (    auth_getoptionenvint("disableinsecurepop3")
+	    && ((p=getenv("POP3_TLS")) == NULL || !atoi(p)))
+	{
+		printed(printf("-ERR POP3 access disabled via insecure connection.\r\n"));
+		fflush(stdout);
+		exit(1);
+	}
+
+	fprintf(stderr, "INFO: LOGIN, user=%s, ip=[%s], port=[%s]\n",
 			authaddr,
-			remoteip);
+					remoteip,
+					remoteport);
 	fflush(stderr);
 
 	msglist_cnt=0;
